package agent

import (
	"context"

	v1alpha1Client "github.com/aquasecurity/starboard/pkg/generated/clientset/versioned/typed/aquasecurity/v1alpha1"
	"gitlab.com/gitlab-org/cluster-integration/gitlab-agent/v14/internal/module/modagent"
	"gitlab.com/gitlab-org/cluster-integration/gitlab-agent/v14/internal/module/starboard_vulnerability"
	"gitlab.com/gitlab-org/cluster-integration/gitlab-agent/v14/pkg/agentcfg"
	"go.uber.org/zap"
	"google.golang.org/protobuf/proto"
	"k8s.io/apimachinery/pkg/util/wait"
)

type module struct {
	log    *zap.Logger
	api    modagent.Api
	client v1alpha1Client.VulnerabilityReportsGetter
}

func (m *module) Run(ctx context.Context, cfg <-chan *agentcfg.AgentConfiguration) error {
	var holder *workerHolder

	defer func() {
		if holder != nil {
			holder.stop()
		}
	}()

	for config := range cfg {
		if holder != nil {
			if holder.configChanged(config) {
				holder.stop()
				holder = m.newWorkerHolder(ctx, config)
			}
		} else {
			holder = m.newWorkerHolder(ctx, config)
		}
	}

	return nil
}

func (m *module) newWorkerHolder(ctx context.Context, cfg *agentcfg.AgentConfiguration) *workerHolder {
	if cfg.Starboard == nil {
		return nil
	}

	config := cfg.Starboard
	holder := &workerHolder{
		config:  config,
		agentID: cfg.AgentId,
	}

	ctx, holder.cancel = context.WithCancel(ctx)
	w := &worker{
		log:     m.log,
		api:     m.api,
		client:  m.client,
		config:  holder.config,
		agentID: holder.agentID,
	}
	holder.wg.StartWithContext(ctx, w.Run)
	return holder
}

func (m *module) DefaultAndValidateConfiguration(cfg *agentcfg.AgentConfiguration) error {
	return nil
}

func (m *module) Name() string {
	return starboard_vulnerability.ModuleName
}

type workerHolder struct {
	agentID int64
	config  *agentcfg.StarboardCF
	wg      wait.Group
	cancel  context.CancelFunc
}

func (h *workerHolder) configChanged(cfg *agentcfg.AgentConfiguration) bool {
	return !proto.Equal(h.config, cfg.Starboard)
}

func (h *workerHolder) stop() {
	h.cancel()
	h.wg.Wait()
}
